<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\NewCode;
use Teknisa\SM\Service\Sm00006000RelacServicoService;

class Sm00006000RelacServicoController extends SMCrud
{
    protected $dataSourceName = 'fct_relacservico_widget';
    protected $newCode;
    protected $ServiceRelacServico;

    public function __construct($crudService, $environment, NewCode $newCode, Sm00006000RelacServicoService $ServiceRelacServico) {
        parent::__construct($crudService, $environment);
        $this->newCode = $newCode;
        $this->ServiceRelacServico = $ServiceRelacServico;
    }
    
    /**
     * A função "validaServicoRelacionado" verifica se o serviço 
     * que está sendo relacionado já existe está relacionadoao serviço.
     * 
     * @param Request\Row $request
     * @param Response $response
     */
    public function validaServicoRelacionado(Request\Row $request, Response $response) {
    
        $row = $request->getRow();
        
        $servOrcaRelac = $row['CDSERVORCARELAC']; 
        $servOrca = $row['CDSERVORCA'];
        $tipoMedida = $row['CDTPMEDIDASERV'];
        
        $ValidaServRelac = $this->ServiceRelacServico->validaServicoRelacionado($servOrcaRelac, $servOrca, $tipoMedida);
        
        $dtsValServRelac = new DataSet('VALIDASERV', array('VALIDA'=>$ValidaServRelac));
        
        $response->addDataSet($dtsValServRelac);
    } 
    
    /**
     * A função "validaServicoRelacionadoUnit" verifica se o serviço 
     * que está sendo relacionado já existe está relacionadoao serviço para inclusão 
     * de um só item.
     */
     
    public function validaServicoRelacionadoUnit(Request\Row $request, Response $response) {
    
        $row = $request->getRow();
        
        $servOrcaRelac = $row['CDSERVORCARELAC']; 
        $servOrca = $row['CDSERVORCA'];
        $tipoMedida = $row['CDTPMEDIDASERV'];
        
        $ValidaServRelac = $this->ServiceRelacServico->validaServicoRelacionadoUnit($servOrcaRelac, $servOrca, $tipoMedida);
        
        $dtsValServRelac = new DataSet('VALIDASERV', array('VALIDA'=>$ValidaServRelac));
        
        $response->addDataSet($dtsValServRelac);
    } 
    
    /**
     * Salva o registro.
     * 
     * @param Request\Row $request
     * @param Response $response
     */
    public function save(Request $request, Response $response) {
        
        $rows = $request->getDataSet()->getRows();
        
        foreach($rows as $row) {
            if ($row["__is_new"]) {
                $row["NRRELACSERVICO"] = $this->newCode->getCodeByTableName("FCT_RELACSERVICO", $this->environment->getNrOrg());
            }
        }
        
        $request->setParameter('dbname', $this->dataSourceName);
        
        parent::save($request, $response);
        $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    }
   /**
     * Deleta o registro.
     * 
     * @param Request\Row $request
     * @param Response $response
     */
    public function delete(Request $request, Response $response) {
        $request->setParameter('dbname', $this->dataSourceName);
        parent::delete($request, $response);
    }
    /**
     * Busca o registro.
     * 
     * @param Request\Row $request
     * @param Response $response
     */
    public function find(Request\Filter $request, Response $response){
        $request->setParameter('dbname', $this->dataSourceName);
        
        parent::find($request, $response);
    }
    /**
     * Salva o registro, inserindo múltiplos serviços de uma vez.
     * 
     * @param Request\Row $request
     * @param Response $response
     */
     
     public function SaveAutomatico(Request\Row $request, Response $response) {
        try {
            $row             = $request->getRow();
            $cdservorcas     = $row["CDSERVORCA"];
            $cdservorcarelac = $row["CDSERVORCARELAC"];
            $cdtpmedidaserv  = $row["CDTPMEDIDASERV"];
            $qtmedidaserv    = $row["QTMEDIDASERV"];
            $idmedidafixa    = $row["IDMEDIDAFIXA"];
            $nrorg           = $this->environment->getNrOrg();
            $mensagem        = array();
            
            $dataset = array();
            foreach ($cdservorcas as $cdservorca) {
                foreach ($cdservorcarelac as $servrelac) {
                    $valido = $this->ServiceRelacServico->validaServicoRelacionadoUnit($servrelac, $cdservorca, $cdtpmedidaserv);
                    if($valido) {
                        array_push($mensagem, $cdservorca);
                    } else {
                        $dataset[] = array(
                            "__is_new"        => true,
                            "NRRELACSERVICO"  => $this->newCode->getCodeByTableName("FCT_RELACSERVICO", $this->environment->getNrOrg()),
                            "CDSERVORCA"      => $cdservorca,
                            "CDSERVORCARELAC" => $servrelac,
                            "QTMEDIDASERV"    => $qtmedidaserv,
                            "CDTPMEDIDASERV"  => $cdtpmedidaserv,
                            "IDMEDIDAFIXA"    => $idmedidafixa
                        );
                    }
                }
            }
            
            $requestNovo = new Request\DataSet(new DataSet($this->dataSourceName, $dataset), "fct_relacservico_widget/save", "POST", $request->getUserId());
            parent::save($requestNovo, $response);
            $servNaoInseridos = array();
            if(!$response->getError() && count($mensagem) > 0) 
                array_push($servNaoInseridos, $this->ServiceRelacServico->getNameServorca($mensagem, $nrorg));
            
            $response->addDataSet(new DataSet("VALIDACAO", $servNaoInseridos));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Busca uma árvore de elementos e replica para ela o valor aplicado em um 
     * campo de fixar medida
    **/
    public function replicarAlteracaoMedidaServ(Request\Row $request, Response $response){
        try{
            $row   = $request->getRow();
            $nrorg = $this->environment->getNrOrg();
            $idElemento = $row["IDELEMENTO"];
            $idMedida   = $row["IDMEDIDAFIXA"];
            $servrelac  = $row["CDSERVORCARELAC"];
            $this->ServiceRelacServico->replicarAlteracaoMedidaServ($idElemento, $idMedida, $nrorg, $servrelac);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}
?>